SystemOrganization addCategory: #Matcher!
SystemOrganization addCategory: #'Matcher-Test'!

TestCase subclass: #MatcherTest
	instanceVariableNames: 'techTicketApp someTicket someTicketId testTech bill workerMatchApp someProject someProjectId pepe'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher-Test'!

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 15:37'!
setUp
	self setUpTicketTests.
	self setUpProjectTests.! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 23:21'!
setUpProjectTests
	| newProject worker |
	
	MatcherSkill setWeight: 0.4.
	MatcherLocation setWeight: 0.6.
	
	someProject := Project newWithDescription: 'Some Project' priority: 1 dueBy: (Date today) createdBy: (MatcherUser fullName: 'Pepe Smith').
	someProject addRequirement: (workerMatchApp getMatchItemByName: 'Coding')."skil"
	someProject addRequirement: (workerMatchApp getMatchItemByName: 'HTML')."skill"
	someProject addRequirement: (workerMatchApp getMatchItemByName: 'ECommerce'). "location"
	someProjectId := workerMatchApp addMatchActivity: someProject.
	
	"Worker Creation"
	pepe := Worker withFirstName: 'Pepe' lastName: 'Skillz' address: '0 Worker Way' city: 'Worker Town' state: 'WA' zip: '22333' phone: '5555551414'.
	pepe assignMatchItem: (workerMatchApp getMatchItemByName: 'Coding').
	pepe assignMatchItem: (workerMatchApp getMatchItemByName: 'HTML').
	workerMatchApp addMatchResource: pepe.! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 22:10'!
setUpTicketTests
	| tech newProject |
	
	MatcherSkill setWeight: 0.7.
	MatcherLocation setWeight: 0.3.
	
	techTicketApp := TechTicketApp default.
	techTicketApp initialize.
	techTicketApp privateClearItems.
	workerMatchApp := WorkerMatchApp default.
	workerMatchApp initialize.
	workerMatchApp privateClearItems.

	tech := (Tech withFirstName: 'Test' lastName: 'Techer' address: '1 Tech Way' city: 'Tech City' state: 'PA' zip: '19191' phone: '5555551212').
	tech assignMatchItem: (techTicketApp getMatchItemByName: 'Philadelphia').
	tech assignMatchItem: (techTicketApp getMatchItemByName: 'Washington DC').
	tech assignMatchItem: (techTicketApp getMatchItemByName: 'Computer Repair').
	tech assignMatchItem: (techTicketApp getMatchItemByName: 'Testing').
	tech assignMatchItem: (techTicketApp getMatchItemByName: 'CCNA').
	tech setAvailability: #(monday wednesday friday) asOrderedCollection.
	testTech := tech.
	techTicketApp addMatchResource: tech.
	
	tech := (Tech withFirstName: 'Empty' lastName: 'Skills' address: '0 Tech Way' city: 'Tech Town' state: 'WA' zip: '22333' phone: '5555551414').
	techTicketApp addMatchResource: tech.
	
	tech := (Tech withFirstName: 'Bill' lastName: 'Lumberg' address: '1 Tech Way' city: 'Tech City' state: 'PA' zip: '19191' phone: '5555551212').
	tech assignMatchItem: (techTicketApp getMatchItemByName: 'Orlando').
	tech assignMatchItem: (techTicketApp getMatchItemByName: 'Computer Repair').
	tech assignMatchItem: (techTicketApp getMatchItemByName: 'Java').
	bill := tech.
	techTicketApp addMatchResource: tech.
	
	someTicket := Ticket newWithDescription: 'Test Ticket' priority: 1 dueBy: (Date today) createdBy: (MatcherUser fullName: 'Webmaster').
	someTicket addRequirement: (techTicketApp getMatchItemByName: 'Testing').
	someTicket addRequirement: (techTicketApp getMatchItemByName: 'Computer Repair').
	someTicket addRequirement: (techTicketApp getMatchItemByName: 'New York City').

	someTicketId := techTicketApp addMatchActivity: (someTicket).! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 21:32'!
testMatchBestWorkerByScore
"Second worker has 0 score in the skills match"
| worker addedTicketNum resultingWorkers |
"see setUp for Pepe and someProject"
worker := Worker withFirstName: 'Juanito' lastName: 'Augenblick' address: '123 Main Street' city: 'Shiny Town' state: 'TX' zip: '12333' phone: '5555555454'. 
worker assignMatchItem: (workerMatchApp getMatchItemByName: 'HTML').
workerMatchApp addMatchResource: worker.

resultingWorkers := workerMatchApp findResourceMatchForActivity: someProject.

Transcript show: (someProject bestMatchDictionary).  Transcript cr.

self assert: resultingWorkers size = 2.

self assert: (someProject matchWithTopScore) getResource fullName = 'Pepe Skillz'.! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 21:33'!
testMatchBestWorkerForProjectOutOfPoolOfWorkers
"Second worker has 0 score in the skills match"
| resultingWorkers |

workerMatchApp addMatchResource:  (Worker withFirstName: 'Juanito' lastName: 'Augenblick' address: '123 Main Street' city: 'Shiny Town' state: 'TX' zip: '12333' phone: '5555555454').

resultingWorkers := workerMatchApp findResourceMatchForActivity: someProject.

Transcript show: (someProject bestMatchDictionary).  Transcript cr.

self assert: resultingWorkers size = 1.! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 21:33'!
testMatchProjectFindNoWorker
| resultingWorkers |

"Worker Creation"
workerMatchApp addMatchResource:  (Worker withFirstName: 'PepeNo' lastName: 'Skillz' address: '0 Worker Way' city: 'Worker Town' state: 'WA' zip: '22333' phone: '5555551414' ).

workerMatchApp removeMatchResource: pepe.

resultingWorkers := workerMatchApp findResourceMatchForActivity:  someProject.

Transcript show: (someProject bestMatchDictionary).  Transcript cr.

self assert: resultingWorkers size = 0.! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 21:33'!
testMatchProjectWithWorker
| resultingWorkers |

resultingWorkers := workerMatchApp findResourceMatchForActivity: someProject.

self assert: resultingWorkers size = 1.! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 21:33'!
testMatchTicketWithTech
| resultingTechs |

resultingTechs := techTicketApp findResourceMatchForActivity: someTicket.

Transcript show: (someTicket bestMatchDictionary).  Transcript cr.

self assert: resultingTechs size = 2.

self assert: (someTicket matchWithTopScore) getResource fullName = 'Test Techer'.

! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 21:56'!
testMatchingScoreAvailableAt

self assert: (testTech isAvailableAt: #(wednesday friday)) .
self deny: (testTech isAvailableAt: #(thursday friday)) .
self deny: (testTech isAvailableAt: #(sunday thursday friday)) .
self assert: (testTech isAvailableAt: #(monday)) .
! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 23:31'!
testMatchingScoreSkills

self assert: 57.14 = (someProject matchScore: pepe) getScore. 
! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 22:08'!
testPersonCreation

| person |

person := MatcherResource withFirstName: 'Test' lastName: 'Techer' address: '1 Tech Way' city: 'Tech City' state: 'PA' zip: '19191' phone: '5555551212'.

self assert: 'Test Techer'  = person fullName.
self assert: '1 Tech Way, Tech City, PA 19191'  = person fullAddress.! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 23:02'!
testTechCreation
| tech |

tech := (Tech withFirstName: 'Test' lastName: 'Techer' address: '1 Tech Way' city: 'Tech City' state: 'PA' zip: '19191' phone: '5555551212').
tech assignMatchItem: (techTicketApp getMatchItemByName: 'Philadelphia').
tech assignMatchItem: (techTicketApp getMatchItemByName: 'Washington DC').
tech assignMatchItem: (techTicketApp getMatchSkillByName: 'Computer Repair' withSkill: 4).
tech assignMatchItem: (techTicketApp getMatchItemByName: 'Testing').
tech assignMatchItem: (techTicketApp getMatchItemByName: 'CCNA').
tech setAvailability: #(monday wednesday friday) asOrderedCollection.

self assert: true = (tech hasMatchItem: (techTicketApp getMatchItemByName: 'CCNA')).

self assert: (tech hasMatchItem: ((techTicketApp getMatchSkillByName: 'Computer Repair' withSkill: 1))).
self deny: (tech hasMatchItem: ((techTicketApp getMatchSkillByName: 'Computer Repair' withSkill: 9))).
! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 22:32'!
testTicketCreation
| newTicket addedTicketNum |

newTicket := Ticket newWithDescription: 'Test Ticket' priority: 1 dueBy: (DateAndTime now) createdBy: (MatcherUser fullName: 'The Creator').

newTicket addRequirement: (techTicketApp getMatchItemByName: 'Computer Repair').
newTicket addRequirement: (techTicketApp getMatchItemByName: 'New York City').

addedTicketNum := techTicketApp addMatchActivity: (newTicket).

Transcript show: techTicketApp reportTicketVolumeByHour. "report test thrown in here"

"self assert: (techTicketApp nextTicketId) = ((techTicketApp ticketCount)+1).
self assert: addedTicketNum = ((techTicketApp nextTicketId)-1).
self assert: (((techTicketApp getTicketById: addedTicketNum) status) getName) = 'Open'."! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 22:45'!
testWorkerCreation

| worker daysAvailable |

daysAvailable := #(monday wednesday friday) asOrderedCollection.

"I removed 'traits' from this test... if we want Traits, go ahead and make a MatcherItem subclass and it should be an easy add'"

worker := Worker withFirstName: 'Test' lastName: 'Techer' address: '1 Tech Way' city: 'Tech City' state: 'PA' zip: '19191' phone: '5555551212'.
worker assignMatchItem: (workerMatchApp getMatchItemByName: 'Java').
worker assignMatchItem: (workerMatchApp getMatchItemByName: 'New York City').

self assert: 'Test Techer'  = worker fullName.
self assert: '1 Tech Way, Tech City, PA 19191'  = worker fullAddress.
self assert: (worker hasMatchItem: (workerMatchApp getMatchItemByName: 'Java')). 
self assert: (worker hasMatchItem: (workerMatchApp getMatchItemByName: 'New York City')).! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 22:48'!
testWorkerMatchApp
| workerOne workerTwo countBeforeAdding |

workerOne := Worker withFirstName: 'John' lastName: 'Rodriguez' address: '123 Mainstreet' city: 'Sunrise' state: 'FL' zip: '33326' phone: '3063933939'.

workerTwo := Worker withFirstName: 'Pepe' lastName: 'Smith' address: '1 Tech Way' city: 'Tech City' state: 'PA' zip: '19191' phone: '5555551212'.

countBeforeAdding:= workerMatchApp resourceCount.

workerMatchApp addMatchResource: workerOne.
workerMatchApp addMatchResource: workerTwo.
self assert:  (workerMatchApp resourceCount) = (countBeforeAdding+2).! !

OrderedCollection subclass: #MatcherItems
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

MatcherItems subclass: #Certifications
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!Certifications methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/1/2013 00:16'!
label
	^'Certifications'! !

MatcherItems subclass: #DaysAvailable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!DaysAvailable methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/1/2013 00:18'!
label
	^'Days Available'! !

MatcherItems subclass: #Locations
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!Locations methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/1/2013 00:16'!
label
	^'Locations'! !

!MatcherItems class methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/8/2013 14:14'!
withCollection: aCollection
	^self new withCollection: aCollection! !

!MatcherItems methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/19/2013 18:08'!
getMatchScore: resourceCriterias forResource: aResource
|totalScore setSize setMatch|

(self hasEqualElements:  resourceCriterias) 
	ifTrue: [^MatcherScore newWithScore: 100 andResource: aResource].

setSize := resourceCriterias size.
setMatch := 0.

(setSize = 0) 
	ifTrue: [^MatcherScore newWithScore: 0 andResource: aResource].

resourceCriterias do: [ :criteria | 
	(self includes: criteria) 
		ifFalse: [
			
			setMatch := setMatch + 1.
			
			]. 
		].
	
	
totalScore := ((setSize - setMatch)*100)/setSize.
	
^MatcherScore newWithScore: ((totalScore * 1.00) rounded) andResource: aResource! !

!MatcherItems methodsFor: 'adding' stamp: 'HectorGonzalez 4/19/2013 13:42'!
withCollection: aCollection
	self addAllLast: aCollection! !

MatcherItems subclass: #Skills
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!Skills methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/1/2013 00:15'!
label
	^'Skills'! !

MatcherItems subclass: #Traits
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!Traits methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/1/2013 00:17'!
label
	^'Traits'! !

Object subclass: #MatcherActivity
	instanceVariableNames: 'id requirements description status creator bestMatchDictionary priority dueByTime assignedPersonsDictionary creationDate lastFetch'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!MatcherActivity class methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 00:39'!
newWithDescription: desc priority: priorityNum dueBy: dueByDate createdBy: user

^ self new newWithDescription: desc priority: priorityNum dueBy: dueByDate createdBy: user

! !

!MatcherActivity methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/19/2013 17:59'!
addBestMatchRun: aMatchDictionary
	"The idea here is to keep track of matches for any given item, i.e. a ticket has a matcher algorithm run on it (see it in TechTicketApp) which then"
	"creates a dictionary of best matches with the mapping Tech->MatcherScore.  This dictionary is then a member of the matchDictionary variable"
	"in this class, which has keys which are the dates they were added/generated so we can see progress over time."
	lastFetch := DateAndTime now.
	bestMatchDictionary at: (lastFetch) put: aMatchDictionary ! !

!MatcherActivity methodsFor: 'as yet unclassified' stamp: 'MattF 4/19/2013 22:41'!
addRequirement: aMatcherItem
	requirements add: aMatcherItem.! !

!MatcherActivity methodsFor: 'as yet unclassified' stamp: 'MattF 4/19/2013 22:47'!
addRequirementList: aMatcherItemOrdList
	aMatcherItemOrdList do: [:ele | requirements add: ele].! !

!MatcherActivity methodsFor: 'as yet unclassified' stamp: 'Mattf 4/13/2013 22:39'!
bestMatchDictionary
	^ bestMatchDictionary! !

!MatcherActivity methodsFor: 'as yet unclassified' stamp: 'MattF 4/16/2013 22:02'!
creationDate
	^ creationDate! !

!MatcherActivity methodsFor: 'as yet unclassified' stamp: 'Mattf 4/13/2013 22:41'!
id
	^ id.! !

!MatcherActivity methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/16/2013 22:16'!
isClosed
	^ (status = MatcherStatus Closed)! !

!MatcherActivity methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/16/2013 22:16'!
isOpen
	^ (status = MatcherStatus Open)! !

!MatcherActivity methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 21:17'!
matchScore: aMatcherResource
"accepts a ticket object and a tech object to provide a score or true/false for how well they match up"
"not sure if this should be in its own class for a good pattern use somewhere"
| tempScore tempTotalWeight |
"note for myself:  I calculate the total weight based on what we see in the MatcherItem classes, so it may not be out of 100"
"but we will end up with a normal percent to keep things normal"
tempScore := 0.
tempTotalWeight := 0.
(requirements size) > 0 
	ifTrue: [requirements do: 
				[:ele | (aMatcherResource hasMatchItem: ele) ifTrue: [tempScore := (tempScore asFloat) + (ele class weight)].  
					tempTotalWeight := tempTotalWeight + ele class weight.]. 
				tempScore :=  (tempScore / tempTotalWeight) * 100.] 
	ifFalse: [tempScore := 100]. 

"The weighting is now set in the class variable weight by MatcherItem type, which could be set up by the user or just on app startup"
"This allows a dynamic list of requirements objects without us having to know what type they are but still get a weighted score"

^ MatcherScore newWithScore: tempScore andResource: aMatcherResource.! !

!MatcherActivity methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/19/2013 17:59'!
matchWithTopScore
|topResource|

(bestMatchDictionary at:lastFetch) do: [:resource|	
		(topResource = nil)
			ifTrue:[
				topResource := resource.
			].		
		(resource getScore) >  (topResource getScore)
			ifTrue:[
				topResource := resource.
			].
	].

^ topResource.! !

!MatcherActivity methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 00:59'!
newWithDescription: desc priority: priorityNum dueBy: dueByDate  createdBy: user


description := desc.
priority := priorityNum.
dueByTime := dueByDate.
status := MatcherStatus Open. 
creator := user.
assignedPersonsDictionary := Dictionary new.
requirements := MatcherItems new.
bestMatchDictionary := Dictionary new.
creationDate := (DateAndTime now).
! !

MatcherActivity subclass: #Project
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!Project methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/16/2013 22:12'!
assignEntrepreneur: anEntrepreneur
	assignedPersonsDictionary at: anEntrepreneur add: Date today.
	anEntrepreneur privateAssignProject: self.! !

!Project methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 23:29'!
matchScore: aWorker
	^ super matchScore: aWorker.
"Didnt want to remove this on you but think the superclass score methodolgy works better for a framework setup"
"
|workerSkills score_obj|
workerSkills := aWorker matchItemList.
score_obj := requirements getMatchScore: workerSkills  forResource: aWorker.

^ score_obj.
"! !

!Project methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 00:55'!
newWithDescription: desc priority: priorityNum dueBy: dueByDate  createdBy: user

id := WorkerMatchApp default nextActivityId.
super newWithDescription: desc priority: priorityNum dueBy: dueByDate createdBy: user.! !

MatcherActivity subclass: #Ticket
	instanceVariableNames: 'completionTime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!Ticket methodsFor: 'as yet unclassified' stamp: 'MattF 4/16/2013 22:06'!
assignTech: aTech
	assignedPersonsDictionary at: aTech add: (DateAndTime now).
	aTech privateAssignTicket: self.! !

!Ticket methodsFor: 'as yet unclassified' stamp: 'Mattf 4/13/2013 21:00'!
closeTicket
	status := MatcherStatus Closed.! !

!Ticket methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 00:55'!
newWithDescription: desc priority: priorityNum dueBy: dueByDate  createdBy: user

id := TechTicketApp default nextActivityId.
super newWithDescription: desc priority: priorityNum dueBy: dueByDate createdBy: user.! !

!Ticket methodsFor: 'as yet unclassified' stamp: 'Mattf 4/13/2013 21:16'!
removeTech: aTech
	assignedPersonsDictionary removeKey: aTech.
	aTech privateRemoveTicket: self.! !

Object subclass: #MatcherHistorical
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!MatcherHistorical methodsFor: 'as yet unclassified' stamp: 'Mattf 4/7/2013 21:58'!
README
"this is a purely a placeholder so I don't forget the idea (if we have time) of adding in historical changes to matcher items like tickets or projects"! !

Object subclass: #MatcherHours
	instanceVariableNames: 'beginAvailable endAvailable onCallBegin onCallEnd'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!MatcherHours methodsFor: 'as yet unclassified' stamp: 'MattF 3/31/2013 12:35'!
withBeginTime: aBeginTime endTime: anEndTime

beginAvailable := aBeginTime .
endAvailable := anEndTime .
onCallBegin := Time midnight.
onCallEnd := Time midnight.! !

!MatcherHours methodsFor: 'as yet unclassified' stamp: 'MattF 3/31/2013 12:35'!
withBeginTime: aBeginTime endTime: anEndTime onCallBeginTime: anOnCallBeginTime onCallEndTime: anOnCallEndTime

beginAvailable := aBeginTime .
endAvailable := anEndTime .
onCallBegin := anOnCallBeginTime.
onCallEnd := anOnCallEndTime.! !

Object subclass: #MatcherItem
	instanceVariableNames: 'name description level createdDate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

MatcherItem class
	instanceVariableNames: 'weight'!

MatcherItem class
	instanceVariableNames: 'weight'!

MatcherItem subclass: #MatcherCertification
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!MatcherItem class methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 16:41'!
setWeight: aNumber
	(aNumber > 1 | aNumber < 0) ifTrue: [self error: 'Weights should be between zero and one.'] ifFalse: [weight := aNumber].! !

!MatcherItem class methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 16:47'!
weight
	^ weight.! !

!MatcherItem class methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 16:46'!
withName: aName andDescription:aDescription
	^ self new withName: aName andDescription: aDescription.! !

!MatcherItem methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 17:18'!
= aMatcherItem
	(self class) = (aMatcherItem class) ifFalse: [^ false].
	(self getName = aMatcherItem getName) ifTrue: [^ true].
	^ false.! !

!MatcherItem methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/8/2013 14:04'!
getDescription
^description! !

!MatcherItem methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/8/2013 14:04'!
getName
^name! !

!MatcherItem methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/19/2013 17:42'!
withName: aName andDescription:aDescription
name:= aName.
description:=aDescription.
createdDate := (DateAndTime now).! !

MatcherItem subclass: #MatcherLocation
	instanceVariableNames: 'address city state additionalInfo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!MatcherLocation class methodsFor: 'as yet unclassified' stamp: 'MattF 3/31/2013 01:02'!
empty

^ (MatcherLocation new) newWithName: 'Unknown' address: '' city: '' state: '' additionalInfo: 'Location has not been set'! !

!MatcherLocation class methodsFor: 'as yet unclassified' stamp: 'MattF 3/31/2013 13:04'!
newWithName: strName address: strAddress city: strCity state: strState additionalInfo: strAdditionalInfo

^ self new newWithName: strName address: strAddress city: strCity state: strState additionalInfo: strAdditionalInfo.! !

!MatcherLocation methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 17:18'!
= aMatcherLocation
	(self class) = (aMatcherLocation class) ifFalse: [^ false].
	(self address = aMatcherLocation address & self city = aMatcherLocation city & self state = aMatcherLocation state) ifTrue: [^ true].
	^ false.! !

!MatcherLocation methodsFor: 'as yet unclassified' stamp: 'Mattf 4/10/2013 21:52'!
address
	^ address! !

!MatcherLocation methodsFor: 'as yet unclassified' stamp: 'Mattf 4/10/2013 21:53'!
city
	^ city! !

!MatcherLocation methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/8/2013 14:18'!
newWithName: aName address: strAddress city: strCity state: strState additionalInfo: strAdditionalInfo
address := strAddress.
city := strCity.
state := strState.
additionalInfo := strAdditionalInfo.

super withName:aName andDescription:''.! !

!MatcherLocation methodsFor: 'as yet unclassified' stamp: 'Mattf 4/10/2013 21:53'!
state
	^ state! !

MatcherItem subclass: #MatcherSkill
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!MatcherSkill class methodsFor: 'as yet unclassified' stamp: 'Mattf 4/7/2013 22:32'!
withName: strName level: numLevel description: strDesc

^ self new withName: strName level: numLevel description: strDesc.! !

!MatcherSkill methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 23:18'!
= aMatcherSkill
"(self getName = 'Computer Repair') ifTrue: [self halt]."
	(self class) = (aMatcherSkill class) ifFalse: [^ false].
	(aMatcherSkill getName = self getName) & (self @ aMatcherSkill) ifTrue: [^ true].
	^ false.! !

!MatcherSkill methodsFor: 'as yet unclassified' stamp: 'MattF 4/19/2013 23:03'!
@ aMatcherSkill
	"Equals is defined in MatcherItem since that can just be name... but this shorthand should help us"
	"tell if a given skill is at or higher than the skill passed to it, hence use of @"
	(self level >= aMatcherSkill level) ifTrue: [^ true].
	^ false.! !

!MatcherSkill methodsFor: 'as yet unclassified' stamp: 'Mattf 4/10/2013 22:34'!
level
	^ level! !

!MatcherSkill methodsFor: 'as yet unclassified' stamp: 'Mattf 4/10/2013 23:22'!
level: aNumber
	level := aNumber.! !

!MatcherSkill methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/8/2013 14:17'!
withName: aName level: numLevel description: aDescription
level := numLevel .
super withName:aName andDescription: aDescription.! !

MatcherItem subclass: #MatcherStatus
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!MatcherStatus class methodsFor: 'as yet unclassified' stamp: 'Mattf 4/13/2013 20:58'!
Closed
	^ self new newWithName: 'Closed'.! !

!MatcherStatus class methodsFor: 'as yet unclassified' stamp: 'Mattf 4/13/2013 20:58'!
Open
	^ self new newWithName: 'Open'.! !

!MatcherStatus class methodsFor: 'as yet unclassified' stamp: 'Mattf 4/7/2013 20:59'!
newWithName: aName
	^ self new newWithName: aName.! !

!MatcherStatus methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/8/2013 14:19'!
newWithName: aName
	"I called this strName because name was overriding some base method in Object, which didn't seem right to do"
super withName:aName andDescription: ''.! !

Object subclass: #MatcherItemResourceStrategy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

MatcherItemResourceStrategy subclass: #LocationMatchOnly
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!MatcherItemResourceStrategy methodsFor: 'as yet unclassified' stamp: 'Mattf 4/11/2013 19:56'!
match: aTech withTicket: aTicket withSkills: skills withLocations: locations withCertifications: certifications
 ^ self subclassResponsibility ! !

MatcherItemResourceStrategy subclass: #SkillLocationMatch
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

Object subclass: #MatcherMainApp
	instanceVariableNames: 'masterMatchItemCollection activityCollection resourceCollection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

MatcherMainApp class
	instanceVariableNames: 'soleInstance'!

MatcherMainApp class
	instanceVariableNames: 'soleInstance'!

!MatcherMainApp methodsFor: 'as yet unclassified' stamp: 'MattF 4/19/2013 23:27'!
activityCount
	^ activityCollection size.! !

!MatcherMainApp methodsFor: 'as yet unclassified' stamp: 'MattF 4/19/2013 23:30'!
addMatchActivity: aMatcherActivity
	activityCollection add: aMatcherActivity.
	^ aMatcherActivity id! !

!MatcherMainApp methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 15:50'!
addMatchItem: aMatcherItem
	masterMatchItemCollection add: aMatcherItem.
! !

!MatcherMainApp methodsFor: 'as yet unclassified' stamp: 'MattF 4/19/2013 23:21'!
addMatchResource: aMatcherResource
	resourceCollection add: aMatcherResource.! !

!MatcherMainApp methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 21:31'!
findResourceMatchForActivity: aMatcherActivity
	"Provides a dictionary of scores, with resources as keys for a given activity"
	| matchesDict checkScore |
	matchesDict  := Dictionary new.
	resourceCollection do: [:techEle | 
		checkScore := (aMatcherActivity matchScore: techEle).  
		(checkScore getScore) = 0 ifFalse: [matchesDict at: techEle put: checkScore ]].
	
	aMatcherActivity addBestMatchRun: matchesDict.
	
	^ matchesDict.
	! !

!MatcherMainApp methodsFor: 'as yet unclassified' stamp: 'MattF 4/19/2013 23:24'!
getMatchActivityById: anIdNumber
	"Returns a ticket based on it's ID number"
	| aTicket |
	^ activityCollection do: [:ele | ((ele id) = anIdNumber) ifTrue: [ ^ ele ] ].! !

!MatcherMainApp methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 01:28'!
getMatchItemByName: aName
	masterMatchItemCollection do: [:ele | (ele getName = aName) 
		ifTrue: [^ ele copy]
		].
	self error: 'Item not found (', aName, '), add it to the system first'..! !

!MatcherMainApp methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 22:27'!
getMatchSkillByName: aName withSkill: aNumber
	| tempSkill |
	masterMatchItemCollection do: [:ele | (ele getName = aName & (ele class asString) = 'MatcherSkill') 
		ifTrue: [tempSkill := ele copy.  tempSkill level: aNumber. ^ tempSkill.]
		].
	self error: 'Item not found (', aName, '), add it to the system first'..! !

!MatcherMainApp methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 00:08'!
initialize
	masterMatchItemCollection := MatcherItems new.
	activityCollection := OrderedCollection new.
	resourceCollection := MatcherItems new.! !

!MatcherMainApp methodsFor: 'as yet unclassified' stamp: 'MattF 4/19/2013 23:43'!
nextActivityId
"Provides the next activity id from the system, so we don't ever re-use an ID"
	| newActivityNum |
	newActivityNum := 1.
	(activityCollection size = 0) ifTrue:[^newActivityNum].
	activityCollection do: [:element | 
		((element = nil) or: (element id) = nil) ifFalse: [ 
			
							((element id) >= newActivityNum)
							 ifTrue: [newActivityNum := (element id) +1]
			
								].
	].
	^ newActivityNum.! !

!MatcherMainApp methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 01:24'!
privateClearItems
	self initialize.! !

!MatcherMainApp methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 15:49'!
removeMatchActivity: aMatcherActivity
	activityCollection remove: aMatcherActivity.
	^ aMatcherActivity id! !

!MatcherMainApp methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 15:51'!
removeMatchItem: aMatcherItem
	masterMatchItemCollection remove: aMatcherItem.
! !

!MatcherMainApp methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 15:49'!
removeMatchResource: aMatcherResource
	resourceCollection remove: aMatcherResource.! !

!MatcherMainApp methodsFor: 'as yet unclassified' stamp: 'MattF 4/19/2013 23:27'!
resourceCount
	^ resourceCollection size.! !

MatcherMainApp subclass: #TechTicketApp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!TechTicketApp class methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/8/2013 11:31'!
default
	(soleInstance isNil) ifTrue: [soleInstance := super new initialize].
	^ soleInstance.! !

!TechTicketApp class methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/8/2013 11:34'!
new
	self error: 'This is a singleton, use default'! !

!TechTicketApp methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 22:26'!
initialize 
"Start up the main app object with some default values for logical items and initialize the others to a base collection"

super initialize .
 
"there should be some default locations I would think...user can take them out if not needed?"
masterMatchItemCollection add: (MatcherLocation newWithName: 'New York City' address: '1 Main St.' city: 'New York' state: 'NY' additionalInfo: 'Main NY Office').
masterMatchItemCollection add: (MatcherLocation newWithName: 'Washington DC' address: '1600 Pennsylvania Ave.' city: 'Washington' state: 'DC' additionalInfo: 'Main DC Office').
masterMatchItemCollection add: (MatcherLocation newWithName: 'San Francisco' address: '100 Lombard St.' city: 'San Francisco' state: 'CA' additionalInfo: 'Main SF Office').
masterMatchItemCollection add: (MatcherLocation newWithName: 'Orlando' address: '876 Disney Pl.' city: 'Orlando' state: 'FL' additionalInfo: 'Main FL Office').
masterMatchItemCollection add: (MatcherLocation newWithName: 'Philadelphia' address: '2 Bell Blvd.' city: 'Philadelphia' state: 'PA' additionalInfo: 'Main PHL Office').

"adding in some base skills... no point in adding a level, that should be user set for either the ticket to assign it to or the tech to assign it to... this will just allow us a base list to show"
masterMatchItemCollection add: (MatcherSkill withName: 'Computer Repair' level: 0 description: 'Ability to fix computers').
masterMatchItemCollection add: (MatcherSkill withName: 'Electrical Repair' level: 0 description: 'Electrical Certification').
masterMatchItemCollection add: (MatcherSkill withName: 'Testing' level: 0 description: 'Apitude as a tester').
masterMatchItemCollection add: (MatcherSkill withName: 'Java' level: 0 description: 'Apitude in Java').
masterMatchItemCollection add: (MatcherSkill withName: 'Teaching' level: 0 description: 'Ability to teach').

masterMatchItemCollection add: (MatcherCertification withName: 'CCNA' andDescription: 'ccna cert').
! !

!TechTicketApp methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 00:04'!
reportTicketVolumeByHour
	| resultsColl |
	resultsColl := OrderedCollection newFrom: (0 to: 24 by: 0.5).
	resultsColl := Dictionary newFromPairs: resultsColl.
	resultsColl keysDo: [:ele | resultsColl at: ele put: 0].
	
	activityCollection do: [:ele | resultsColl at: ((ele creationDate) hour24) put: ((resultsColl at: ((ele creationDate) hour24))+1)].
	^ resultsColl 
	
	! !

MatcherMainApp subclass: #WorkerMatchApp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!WorkerMatchApp class methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 00:30'!
default
	(soleInstance isNil) ifTrue: [soleInstance := super new initialize].
	^ soleInstance.! !

!WorkerMatchApp class methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/8/2013 11:34'!
new
	self error: 'This is a singleton, use default'! !

!WorkerMatchApp methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 22:44'!
initialize 

super initialize .

masterMatchItemCollection add: (MatcherSkill withName: 'HTML' level: 0 description: 'Ability to write HTML').
masterMatchItemCollection add: (MatcherSkill withName: 'Java' level: 0 description: 'Ability to write Java').
masterMatchItemCollection add: (MatcherSkill withName: 'CSS' level: 0 description: 'Ability to write CSS').
masterMatchItemCollection add: (MatcherSkill withName: 'PHP' level: 0 description: 'Ability to write PHP').
masterMatchItemCollection add: (MatcherSkill withName: 'Coding' level: 0 description: 'Ability to Code').

masterMatchItemCollection add: (MatcherLocation newWithName: 'New York City' address: '1 Main St.' city: 'New York' state: 'NY' additionalInfo: 'Main NY Office').
masterMatchItemCollection add: (MatcherLocation newWithName: 'ECommerce' address: 'N/A' city: 'N/A' state: '' additionalInfo: 'Online Store').
! !

Object subclass: #MatcherResource
	instanceVariableNames: 'id matchItemList matchActivityList fname lname address city state zip phone description daysAvailable'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!MatcherResource class methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 00:34'!
withFirstName: strFName lastName: strLName address: strAddress city: strCity state: strState zip: strZip phone: strPhone

^ self new withFirstName: strFName lastName: strLName address: strAddress city: strCity state: strState zip: strZip phone: strPhone! !

!MatcherResource methodsFor: 'as yet unclassified' stamp: 'MattF 4/19/2013 23:11'!
assignMatchItem: aMatcherItem
	matchItemList add: aMatcherItem.! !

!MatcherResource methodsFor: 'accessing' stamp: 'HectorGonzalez 3/31/2013 20:57'!
fullAddress
	^ (address, ', ', city, ', ', state, ' ', zip)! !

!MatcherResource methodsFor: 'accessing' stamp: 'HectorGonzalez 3/31/2013 20:48'!
fullName
  ^(fname, ' ', lname)! !

!MatcherResource methodsFor: 'as yet unclassified' stamp: 'MattF 4/19/2013 23:00'!
hasMatchItem: aMatchItem
	^(matchItemList includes: aMatchItem).! !

!MatcherResource methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/8/2013 14:23'!
id
	^ id.! !

!MatcherResource methodsFor: 'testing' stamp: 'MattF 4/20/2013 21:54'!
isAvailableAt: theseDays
	"Didnt think this needed a score, just a true or false really..."
	theseDays do: [:ele | (daysAvailable includes: ele) ifFalse: [^ false]].
	^ true.
	
	"^(daysAvailable getMatchScore:  theseDays forResource: self)"! !

!MatcherResource methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 00:07'!
matchItemList
	^ matchItemList ! !

!MatcherResource methodsFor: 'testing' stamp: 'Mattf 4/13/2013 22:44'!
printOn: aStream
	self fullName printOn: aStream.! !

!MatcherResource methodsFor: 'testing' stamp: 'MattF 4/20/2013 01:07'!
privateAssignActivity: aMatcherActivity
	matchActivityList at: aMatcherActivity put: (DateAndTime now)! !

!MatcherResource methodsFor: 'testing' stamp: 'MattF 4/20/2013 01:08'!
privateRemoveActivity: aMatcherActivity
	matchActivityList removeKey: aMatcherActivity! !

!MatcherResource methodsFor: 'testing' stamp: 'MattF 4/20/2013 21:35'!
setAvailability: aCollection
	daysAvailable addAll: aCollection.! !

!MatcherResource methodsFor: 'initialize-release' stamp: 'MattF 4/20/2013 21:51'!
withFirstName: strFName lastName: strLName address: strAddress city: strCity state: strState zip: strZip phone: strPhone
	
fname := strFName.
lname := strLName.
address := strAddress.
city := strCity.
state := strState.
zip := strZip.
phone := strPhone.
matchItemList := OrderedCollection new.

daysAvailable := OrderedCollection new.! !

MatcherResource subclass: #Tech
	instanceVariableNames: 'currentLocation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!Tech methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 21:29'!
withFirstName: strFName lastName: strLName address: strAddress city: strCity state: strState zip: strZip phone: strPhone

super 		withFirstName: strFName 
			lastName: strLName 
			address: strAddress 
			city: strCity 
			state: strState 
			zip: strZip 
			phone: strPhone.

currentLocation := nil.
! !

MatcherResource subclass: #Worker
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!Worker class methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/1/2013 00:09'!
withSkillsCollection: skillsArr locationCollection: locationArr firstName: strFName lastName: strLName address: strAddress city: strCity state: strState zip: strZip phone: strPhone traits: traitsArr daysAvailable: aSetOfDaysAvailable

^ self new withSkillsCollection: skillsArr locationCollection: locationArr firstName: strFName lastName: strLName address: strAddress city: strCity state: strState zip: strZip phone: strPhone traits: traitsArr daysAvailable: aSetOfDaysAvailable.

! !

Object subclass: #MatcherScore
	instanceVariableNames: 'score fullMatch resource'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

MatcherScore class
	instanceVariableNames: 'scaleDictionary'!

MatcherScore class
	instanceVariableNames: 'scaleDictionary'!

!MatcherScore class methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/19/2013 18:02'!
newWithScore: aNumber andResource:aResource
	^ self new newWithScore: aNumber andResource: aResource.! !

!MatcherScore methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/19/2013 18:01'!
getResource
	^resource! !

!MatcherScore methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/8/2013 10:43'!
getScore
	^score! !

!MatcherScore methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/8/2013 13:33'!
isFullMatch
	^(score = 100).! !

!MatcherScore methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/8/2013 13:32'!
newWithScore: aNumber
	score := aNumber.! !

!MatcherScore methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 23:31'!
newWithScore: aNumber andResource: aResource
	score := aNumber roundTo: 0.01.
	resource := aResource.! !

!MatcherScore methodsFor: 'as yet unclassified' stamp: 'Mattf 4/13/2013 22:43'!
printOn: aStream
	score printOn: aStream.! !

Object subclass: #MatcherUser
	instanceVariableNames: 'fullName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!MatcherUser class methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/7/2013 17:48'!
fullName: aFullName
	^self new fullName: aFullName! !

!MatcherUser methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/7/2013 17:49'!
fullName: aFullName
fullName:=aFullName! !
